%% Demo to process a source separation in the frequency domain
close all;clear;clc;

ltfatstart;

addpath('data/wav');
addpath('data/filtres');
addpath('inverse');
addpath('utils');

%% First create a mixture of N source signals
N = 4; %number of sources
T = 2^10; %length of sources
S = zeros(N,T); %Matrix of original sources

s = audioread('mix11_s1.wav');
S(1,:) =  s(1:T)';
s = audioread('mix11_s2.wav');
S(2,:) =  s(1:T)';
s = audioread('mix11_s3.wav');
S(3,:) =  s(1:T)';
s = audioread('mix12_s1.wav');
S(4,:) =  s(1:T)';

reverb_time = '250ms'; %filters parameter: reverberation time
dist = '1m'; %filters paramater: distance between mic
nbsrc =  int2str(N); %filters paramater: number of sources
load([nbsrc 'sources_' reverb_time '_' dist '_filt' '.mat']); %load corresponding stereo filtering system in A

X = creation_mel_conv(A,S);%stereo mixture of the source signals with A


%% Set the STFT and iSTFT parameters
L_win = 1024;
a = L_win/2;
g = gabwin({'tight', 'hann'}, a, L_win);

TT = size(X,2);
op.analysis = @(x) dgtreal(x,g,a,L_win);
op.synthesis = @(x) idgtreal(x,g,a,L_win,TT);

%% source separation


% with N-L1 under exact reconstruction
L_win = 1024; %size of the stft window
[S_lp] =  minlp(X,A,1,L_win,op);
S_lp = S_lp(:,1:T);
[SDRlp,SIRlp,SARlp] = bsseval(S_lp(:,1:T),S,512);% performance measurement





% with duet
Se=sep_freq(X,A,L_win,op);
[SDRd,SIRd,SARd] = bsseval(Se(:,1:T),S,512); % performance measurement


% options for the algorithm
options.L_win = L_win; %size of the stft window

options.penalty = 'l1'; % we use Lasso, can be l12 for E-Lasso
options.maxit = 20000; %number of iterations

% compute lipschitz constant
W = options.L_win;
[M,N,K] = size(A);
Wd2 = W/2;
Nf = Wd2+1;
Tf=ceil(TT/Wd2);
fmat=exp(-2*1i*pi/W*(0:K-1).'*(0:Wd2));
Af=reshape(reshape(A,2*N,K)*fmat,2,N,Nf);
nAT = conj(permute(Af,[2,1,3]));
prod1 = zeros(N,N,Nf);
normP = zeros(Nf,1);
for k =1:Nf
    prod1(:,:,k) = nAT(:,:,k)*Af(:,:,k);
    normP(k) = norm(prod1(:,:,k));
end
C = max(normP);
options.lipschitz = C;


% end options

lambda_vect = [1e-8,1e-7,5e-6,1e-6,5e-5,1e-5,1e-4,1e-3,1e-2,1e-1];  
lambda_vect=lambda_vect(end:-1:1); %various lambda hyper parameters

nbxp = length(lambda_vect); 
Se = zeros(N,T,nbxp); %will contains estimated sources
SDRse = zeros(N,nbxp); % SDR
SIRse = zeros(N,nbxp); % SIR
SARse = zeros(N,nbxp); % SAR
timexp = zeros(nbxp,1); % executing time
tr = op.analysis(X(1,:)); % to obtain right size of the matrix stft coef.
[Mtr,Ntr] = size(tr);
dcc =  zeros(Mtr,Ntr,N);%dcc will contains stft coefficients

k=0;

for ll =  lambda_vect
  k=k+1;
  tic
  options.init = dcc; %warm start
  [Sit,dcc] = inverse_freq(X,A,ll,op,options);%process the separation
  timexp(k) = toc;
  Se(:,:,k) = Sit;
  [SDRse(:,k),SIRse(:,k),SARse(:,k)] = bsseval(Sit,S,512);%performance measurement
end


