function [S,dccS] = inverse_freq(X,A,lambda,op,options)


[M,N,K] = size(A);
Tx = size(X,2);
T = Tx - K + 1;

if ~isfield(options, 'penalty')
    options.penalty = 'l12';
end

if ~isfield(options, 'maxit')
        options.maxit = 10000;
end


%% Some initializations
W = options.L_win;
Wd2 = W/2;
Nf = Wd2+1;
Tf=ceil(Tx/Wd2);
fmat=exp(-2*1i*pi/W*(0:K-1).'*(0:Wd2));
Af=reshape(reshape(A,2*N,K)*fmat,2,N,Nf);

if ~isfield(options, 'init')
    options.init = zeros(Nf,Tf,N);
end
dccS0 = options.init;

if ~isfield(options, 'lipschitz')
  %% Choice of C: square of the upper bound norm of the operator.
  if K==1 % Instantaneous mixture
    C = norm(A,'fro')^2 + eps*10^8;
  else
    nAT = conj(permute(Af,[2,1,3]));  
    prod1 = zeros(N,N,Nf);
    normP = zeros(Nf,1);  
    for k =1:Nf
      prod1(:,:,k) = nAT(:,:,k)*Af(:,:,k);
      normP(k) = norm(prod1(:,:,k));
    end    
    C = max(normP);
  end
  options.lipschitz = C;
end


dccX=zeros(Nf,Tf,2);
for n=1:M
    tr = op.analysis(X(n,:));
    dccX(:,:,n) = tr;
end

AdccX = zeros(Nf,Tf,N);
for ff=1:Nf
    Aftmp = squeeze(Af(:,:,ff));
    dccXtmp = permute(dccX(ff,:,:),[3,2,1]);
    dccTmp = Aftmp'*dccXtmp;
    dccTmp = reshape(dccTmp,1,N,Tf);
    AdccX(ff,:,:) =   permute(dccTmp,[1 3 2]);  
end

dccS = dccS0;

iC = 1/options.lipschitz;
y = dccS0;


%% Main loop: fista iterations  
for it=1:options.maxit
    old = dccS;
    
    dccS = y - iC*compute_grad_freq_aux(y);
    dccS = prox(dccS,lambda*iC,options.penalty);
        
    y = dccS + (it-1)/(it+4) * (dccS-old);

    if norm(old(:)-dccS(:))/(norm(dccS(:))+eps) < 1e-4
        break;
    end
    
end


%% Back to the time domain
S = zeros(N,T);
for k=1:N
    tr=   op.synthesis(dccS(:,:,k));
    S(k,:) = tr(1:T)';
end




function [grad] = compute_grad_freq_aux(z)

AtAz = zeros(Nf,Tf,N);

 for f=1:Nf
      Aftmp = squeeze(Af(:,:,f));
      ztmp = permute(z(f,:,:),[3,2,1]);
      Aztmp = Aftmp*ztmp;
      AtAztmp = Aftmp'*Aztmp;     
      AtAztmp = reshape(AtAztmp,1,N,Tf);
      AtAz(f,:,:) = permute(AtAztmp,[1 3 2]);
 end
 
 grad =   AtAz - AdccX;
end

end
