function [S,crit]=sparsopt(X,A,alpha)

% Trouve l'optimum de sum(abs(S).^alpha) sous la contrainte que A*S=X par
% recherche exhaustive d'un point initial et optimisation non contrainte
%
% Environ .02s de calcul par point pour alpha>1, .0004s pour alpha<.74, et
% de .01s  .3s pour .74<alpha<1, selon qu'il y a un maximum local ou pas
%
% Exemple de Winter avec
% alpha=1;
% X=[1; .5];
% A=[1 .6 4/sqrt(17); 0 .8 (.8+i*.6)/sqrt(17)];
% on trouve
% S=[0.2267+0.0404i; 0.5110-0.0911i; 0.4811+0.0147i];
% minimum global avec aucune source  zro pour alpha >= .9779

if alpha >= 1
    % Le problme est convexe
    % Optimisation du critre avec initialisation par le pseudo-inverse
    [S,crit]=sparsopt_unc(X,A,alpha,pinv(A)*X);
    
elseif alpha <= 2*log(1.5)/log(3)
    % Le problme comprend des singularits dont l'une est solution
    % Calcul du critre aux singularits et slection du minimum
    S=zeros(3,3);
   % S([1 2],1)=inv(A(:,[1 2]))*X;
   % S([2 3],2)=inv(A(:,[2 3]))*X;
   % S([3 1],3)=inv(A(:,[3 1]))*X;
	
    S([1 2],1)=(A(:,[1 2]))\X;
    S([2 3],2)=(A(:,[2 3]))\X;
    S([3 1],3)=(A(:,[3 1]))\X;
	
    crit=sum(abs(S).^alpha);
    [crit,ind]=min(crit);
    S=S(:,ind);
    
else
    % Le problme comprend des singularits qui ne sont pas toujours solution
    % Calcul du critre sur une grille triangulaire dlimite par les singularits
    S0=pinv(A)*X; A0=null(A);
    coeff=barygrid(50);
    D=-(S0./A0).'*coeff;
    S=S0*ones(1,1275)+A0*D;
    crit=sum(abs(S).^alpha);
    
    % Dtection et optimisation du minimum global ventuel (pas besoin de
    % marge d'erreur due au sampling exprimentalement)
    [crit,ind]=min(crit);
    S=S(:,ind);
    if all(ind ~= [1 50 1275])
        [S,crit]=sparsopt_unc(X,A,alpha,S);
    end
end

return;


function coeff=barygrid(nbpoints)

% Construit une grille de coefficients barycentriques entre 0 et 1
% (taille nbpoints*(nbpoints+1)/2)

coeff=zeros(3,nbpoints*(nbpoints+1)/2);
pos=0;
for p=0:nbpoints-1
    coeff(:,pos+1:pos+nbpoints-p)=[p*ones(1,nbpoints-p);(0:nbpoints-1-p);(nbpoints-1-p:-1:0)];
    pos=pos+nbpoints-p;
end
coeff=coeff/(nbpoints-1);

return;