function [S,crit]=sparsopt_unc(X,A,alpha,S)

% Trouve l'optimum de sum(abs(S).^alpha) sous la contrainte que A*S=X par
% optimisation non contrainte
%
% Initialise S par la valeur propose, sinon tire une valeur de S qui
% respecte la contrainte

% Initialisation
J=size(A,2);
S0=pinv(A)*X;
A0=null(A);
if nargin == 4
    D=pinv(A0)*(S-S0);
else
    D=randn(J-2,1)+1i*randn(J-2,1);
end

% Transformation en rels
S0=[real(S0); imag(S0)];
A0=[real(A0) -imag(A0); imag(A0) real(A0)];
D=[real(D); imag(D)];

% Optimisation
%options=optimset('GradObj','on','Hessian','on','TolFun',0,'TolX',1e-6,'LargeScale','on','Display','off');
options=optimset('GradObj','on','TolFun',0,'TolX',1e-6,'LargeScale','on','Display','off');
[D,crit]=fminunc(@(D) spars(J,D,alpha,S0,A0),D,options);

% Retour aux complexes
S=S0+A0*D;
S=S(1:J)+1i*S(J+1:2*J);

% Passage  la dimension infrieure si blocage dans une singularit de
% dimension suprieure  1
if (J >= 4) && (alpha <= 1) && any(abs(S) < 1e-5)
    [~,ind]=min(abs(S));
    Si=S;
    S=zeros(J,1);
    [S([1:ind-1 ind+1:J]),crit]=sparsopt_unc(X,A(:,([1:ind-1 ind+1:J])),alpha,Si([1:ind-1 ind+1:J]));
end

return;


function [crit,grad,hess]=spars(J,D,alpha,S0,A0)

% Calcul du critre optimis
S=S0+A0*D;
norm=sum(reshape(S,J,2).^2,2);
crit=sum(norm.^(alpha/2));

% Calcul du gradient
if nargout > 1
    fact1=reshape(norm.^(alpha/2-1)*ones(1,2),2*J,1);
    gradS=alpha*S.*fact1;
    grad=A0.'*gradS;
    
    % Calcul de la hessienne
    if nargout > 2
        fact2=norm.^(alpha/2-2);
        cen=alpha*fact1+alpha*(alpha-2)*S.^2.*reshape(fact2*ones(1,2),2*J,1);
        crossprod=prod(reshape(S,J,2),2);
        off=alpha*(alpha-2)*crossprod.*fact2;
        hessS=diag(off,-J)+diag(cen)+diag(off,J);
        hess=zeros(2*J-4);
        for j=1:2*J-4
            hess(j,j)=sum(sum(hessS.*(A0(:,j)*A0(:,j).')));
            for k=1:j-1
                hess(j,k)=sum(sum(hessS.*(A0(:,j)*A0(:,k).')));
                hess(k,j)=hess(j,k);
            end
        end
    end
end

return;